@@ -2295,6 +2295,9 @@ create_lineplot_box (void)
 		b = gtk_hbox_new (FALSE, GNOME_PAD);
 		gtk_box_pack_start (GTK_BOX (box), b, FALSE, FALSE, 0);
 
+		gtk_box_pack_start (GTK_BOX (b),
+				    gtk_label_new ("y="), FALSE, FALSE, 0);
+
 		plot_entries[i] = gtk_entry_new ();
 		g_signal_connect (G_OBJECT (plot_entries[i]), "activate",
 				  entry_activate, NULL);
@@ -3160,7 +3163,7 @@ gel_add_graph_functions (void)
 	GelEFunc *f;
 	GelToken *id;
 
-	new_category ("plotting", _("Plotting"));
+	new_category ("plotting", N_("Plotting"), TRUE /* internal */);
 
 	/* FIXME: add more help fields */
 #define FUNC(name,args,argn,category,desc) \
@@ -3192,9 +3195,9 @@ gel_add_graph_functions (void)
 	/* bogus value */ \
 	d_addfunc_global (d_makevfunc (id, gel_makenum_null()));
 
-	VFUNC (LinePlot, 2, "", "plotting", _("Plot a function with a line.  First come the functions (up to 10) then optionally limits as x1,x2,y1,y2"));
-	VFUNC (SurfacePlot, 2, "", "plotting", _("Plot a surface function which takes either two arguments or a complex number.  First comes the function then optionally limits as x1,x2,y1,y2,z1,z2"));
+	VFUNC (LinePlot, 2, "", "plotting", N_("Plot a function with a line.  First come the functions (up to 10) then optionally limits as x1,x2,y1,y2"));
+	VFUNC (SurfacePlot, 2, "", "plotting", N_("Plot a surface function which takes either two arguments or a complex number.  First comes the function then optionally limits as x1,x2,y1,y2,z1,z2"));
 
-	PARAMETER (LinePlotWindow, _("Line plotting window (limits) as a 4-vector of the form [x1,x2,y1,y2]"));
-	PARAMETER (SurfacePlotWindow, _("Surface plotting window (limits) as a 6-vector of the form [x1,x2,y1,y2,z1,z2]"));
+	PARAMETER (LinePlotWindow, N_("Line plotting window (limits) as a 4-vector of the form [x1,x2,y1,y2]"));
+	PARAMETER (SurfacePlotWindow, N_("Surface plotting window (limits) as a 6-vector of the form [x1,x2,y1,y2,z1,z2]"));
 }
